package com.wincornixdorf.vnc.gui;

//
//  Copyright (C) 2002 Constantin Kaplinsky.  All Rights Reserved.
//
//  This is free software; you can redistribute it and/or modify
//  it under the terms of the GNU General Public License as published by
//  the Free Software Foundation; either version 2 of the License, or
//  (at your option) any later version.
//
//  This software is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU General Public License for more details.
//
//  You should have received a copy of the GNU General Public License
//  along with this software; if not, write to the Free Software
//  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307,
//  USA.
//

//
// Recording frame. It allows to control recording RFB sessions into
// FBS (FrameBuffer Stream) files.
//


import java.awt.BorderLayout;
import java.awt.Color;
import java.awt.Dimension;
import java.awt.FileDialog;
import java.awt.FlowLayout;
import java.awt.GridBagConstraints;
import java.awt.GridBagLayout;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.KeyEvent;
import java.awt.event.WindowEvent;
import java.awt.event.WindowListener;
import java.io.File;

import javax.swing.JButton;
import javax.swing.JFrame;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JTextField;

import com.wincornixdorf.vnc.global.VNCViewer;

public class JRecordingFrame extends JFrame implements WindowListener, ActionListener {
    private static final long serialVersionUID = 1476458973435896620L;

    boolean recording;

    JTextField fnameField;
    JButton browseButton;

    JLabel statusLabel;

    JButton recordButton, nextButton, closeButton;
    VNCViewer viewer;

    //
    // Check if current security manager allows to create a
    // JRecordingFrame object.
    //

    public static boolean checkSecurity() {
        SecurityManager security = System.getSecurityManager();
        if (security == null) return true;

        try {
            security.checkPropertyAccess("user.dir");
            security.checkPropertyAccess("file.separator");
            // Work around (rare) checkPropertyAccess bug
            System.getProperty("user.dir");
        } catch (SecurityException e) {
            System.out.println("SecurityManager restricts session recording.");
            return false;
        }

        return true;
    }

    //
    // Constructor.
    //

    public JRecordingFrame( VNCViewer v) {
        super("TightVNC Session Recording");

        viewer = v;

        // Determine initial filename for next saved session.
        // FIXME: Check SecurityManager.

        String fname = nextNewFilename( System.getProperty("user.dir") + System.getProperty("file.separator") +
                    "vncsession.fbs");

        // Construct new panel with file name field and "Browse" button.

        JPanel jp = new JPanel();
        GridBagLayout gb = new GridBagLayout();
        GridBagConstraints c = new GridBagConstraints();
        jp.setLayout( gb);

        c.weightx = 1.0;
        c.fill = GridBagConstraints.BOTH;
        c.gridwidth = GridBagConstraints.REMAINDER;
        JLabel lHelp = new JLabel( "File name to save next recorded session in:");
        gb.setConstraints( lHelp, c);
        jp.add( lHelp);


        c.gridwidth = GridBagConstraints.RELATIVE;
        fnameField = new JTextField( fname);
        fnameField.addActionListener( this);
        gb.setConstraints( fnameField, c);
        jp.add( fnameField);

        c.gridwidth = GridBagConstraints.REMAINDER;
        browseButton = new JButton( "Browse");
        browseButton.addActionListener( this);
        browseButton.setMnemonic( KeyEvent.VK_B);
        browseButton.setPreferredSize( new Dimension( 100, 25));
        gb.setConstraints( browseButton, c);
        jp.add( browseButton);


        c.gridwidth = GridBagConstraints.REMAINDER;
        statusLabel = new JLabel( "");
        gb.setConstraints( statusLabel, c);
        jp.add( statusLabel);


        JPanel jpBtns = new JPanel( new FlowLayout( FlowLayout.RIGHT, 3, 3));

        recordButton = new JButton( "Record");
        recordButton.addActionListener(this);
        recordButton.setMnemonic( KeyEvent.VK_R);
        jpBtns.add( recordButton);

        nextButton = new JButton( "Next file");
        nextButton.addActionListener( this);
        nextButton.setMnemonic( KeyEvent.VK_N);
        jpBtns.add( nextButton);

        closeButton = new JButton( "Ok");
        closeButton.addActionListener( this);
        closeButton.setMnemonic( KeyEvent.VK_O);
        jpBtns.add( closeButton);


        setLayout( new BorderLayout( 3, 3));
        add( jp,		BorderLayout.CENTER);
        add( jpBtns,	BorderLayout.SOUTH);

        // Set correct text, font and color for the statusLabel.
        stopRecording();

        pack();

        addWindowListener( this);
    }

    //
    // If the given string ends with ".NNN" where NNN is a decimal
    // number, increase this number by one. Otherwise, append ".001"
    // to the given string.
    //

    protected String nextFilename(String fname) {
        int len = fname.length();
        int suffixPos = len;
        int suffixNum = 1;

        if (len > 4 && fname.charAt(len - 4) == '.') {
            try {
                suffixNum = Integer.parseInt(fname.substring(len - 3, len)) + 1;
                suffixPos = len - 4;
            } catch (NumberFormatException e) { }
        }

        char[] zeroes = {'0', '0', '0'};
        String suffix = String.valueOf(suffixNum);

        if (suffix.length() < 3)
            suffix = new String(zeroes, 0, 3 - suffix.length()) + suffix;

        return fname.substring(0, suffixPos) + '.' + suffix;
    }

    //
    // Find next name of a file which does not exist yet.
    //

    protected String nextNewFilename(String fname) {
        String newName = fname;
        File f;
        try {
            do {
                newName = nextFilename(newName);
                f = new File(newName);
            } while (f.exists());
        } catch (SecurityException e) { }

        return newName;
    }

    //
    // Let the user choose a file name showing a FileDialog.
    //

    protected boolean browseFile() {
        File currentFile = new File(fnameField.getText());

        FileDialog fd = new FileDialog(this, "Save next session as...", FileDialog.SAVE);
        fd.setDirectory(currentFile.getParent());
        fd.setVisible(true);

        if (fd.getFile() != null) {
            String newDir = fd.getDirectory();
            String sep = System.getProperty("file.separator");
            if (newDir.length() > 0) {
                if (!sep.equals(newDir.substring(newDir.length() - sep.length())))
                    newDir += sep;
            }

            String newFname = newDir + fd.getFile();
            if (newFname.equals(fnameField.getText())) {
                fnameField.setText(newFname);
                return true;
            }
        }
        return false;
    }

    //
    // Start recording.
    //

    public void startRecording() {
        statusLabel.setText("Status: Recording...");
        statusLabel.setForeground(Color.red);
        recordButton.setText("Stop recording");

        recording = true;

//         viewer.setRecordingStatus(fnameField.getText());
    }

    //
    // Stop recording.
    //

    public void stopRecording() {
        statusLabel.setText("Status: Not recording.");
        statusLabel.setForeground(Color.black);
        recordButton.setText("Record");

        recording = false;

//         viewer.setRecordingStatus(null);
    }

    //
    // Close our window properly.
    //

    public void windowClosing(WindowEvent evt) {
        setVisible(false);
    }

    //
    // Ignore window events we're not interested in.
    //

    public void windowActivated(WindowEvent evt) {}
    public void windowDeactivated (WindowEvent evt) {}
    public void windowOpened(WindowEvent evt) {}
    public void windowClosed(WindowEvent evt) {}
    public void windowIconified(WindowEvent evt) {}
    public void windowDeiconified(WindowEvent evt) {}

    //
    // Respond to button presses
    //
    public void actionPerformed(ActionEvent evt) {

        if ( evt.getSource() == browseButton) {
            if ( browseFile() && recording)
                startRecording();
        }

        if ( evt.getSource() == recordButton) {
            if ( !recording) {
                startRecording();
            } else {
                stopRecording();
                fnameField.setText(nextNewFilename( fnameField.getText()));
            }
        }

        if ( evt.getSource() == nextButton) {
            fnameField.setText(nextNewFilename(fnameField.getText()));
            if ( recording)
                startRecording();
        }

        if ( evt.getSource() == closeButton)
            setVisible(false);
    }
}
