package com.wincornixdorf.vnc.gui;

import java.awt.BorderLayout;
import java.awt.Color;
import java.awt.Cursor;
import java.awt.Dimension;
import java.awt.Graphics;
import java.awt.Image;
import java.awt.Point;
import java.awt.Rectangle;
import java.awt.Toolkit;
import java.awt.event.KeyEvent;
import java.awt.event.KeyListener;
import java.awt.event.MouseEvent;
import java.awt.event.MouseListener;
import java.awt.event.MouseMotionListener;
import java.awt.event.MouseWheelEvent;
import java.awt.event.MouseWheelListener;
import java.awt.geom.Rectangle2D;
import java.awt.image.FilteredImageSource;
import java.awt.image.ImageFilter;
import java.awt.image.ImageProducer;
import java.awt.image.MemoryImageSource;
import java.util.Arrays;
import java.util.Date;
import java.util.Observable;
import java.util.Observer;

import javax.swing.GrayFilter;
import javax.swing.Scrollable;
import javax.swing.SwingUtilities;

import com.wincornixdorf.vnc.rfb.RFBConnJob;
import com.wincornixdorf.vnc.rfb.RFBHandler;
import com.wincornixdorf.vnc.rfb.RFBHandlerEvent;
import com.wincornixdorf.vnc.rfb.conf.SettingsFB;
import com.wincornixdorf.vnc.rfb.conf.SettingsFB.CursorDisplayMode;
import com.wincornixdorf.vnc.rfb.fb.RemoteCursor;
import com.wincornixdorf.vnc.rfb.fb.VNCEncoding;

/**
 * The remote Desktop Panel is responsible for creating a graphical
 * representation of the RFB Data read using a VNCConnection object. This class
 * unlike it's former counterpart the VncCanvas does have no hand whatsoever in
 * extracting nor in storing the RFB images. At the most, it is supposed to keep an
 * Image object with the pixel data to be displayed, scaled to its needs as a
 * private member. Its second purpose is to handle Mouse and Keyboard events which
 * it has to pass to the RFBHandler so they can be transferred to the server.
 */
public class RemoteDesktopPanel extends RFBHandlerJPanel
        implements MouseListener, MouseMotionListener,
        KeyListener, Observer, Scrollable, MouseWheelListener {
    private static final long serialVersionUID = 1719431101003943847L;

    /**
     * Max Time allowed to pass between two distinct ALT key press events
     * to be treated as an "ALT-double tap" prompting the ALT key to be jammed. (in milliseconds)
     */
    private static final int ALT_LOCK_THRESHOLD = 200;

    private Image rDesktop = null;
    private Image rGraytop = null;
    private Image rPointer = null;

    private Rectangle rMouseNow = new Rectangle();

    private Point pMouseDbg = new Point(0, 0);

    private boolean isPreview = false;

    private ImageFilter filter = new GrayFilter(true, 50);
    private ImageProducer producer = null;

    private long lastAltKey = 0;

    public VNCSessionPoolPanel thePoolPanel = null;

    private boolean showControlNavigator = false;

    public RemoteDesktopPanel(boolean preview) {
        super(new BorderLayout());
        isPreview = preview;

        Date d = new Date();
        lastAltKey = d.getTime();

        if (preview) {
            setMinimumSize(new Dimension(120, 90));
            setMaximumSize(new Dimension(160, 120));
        } else {
            //setPreferredSize( getSize());
            addMouseListener(this);
            addMouseMotionListener(this);
            addKeyListener(this);
            setFocusTraversalKeysEnabled(false);
            addMouseWheelListener(this);
        }
    }

    @Override
    public void setConnection(RFBHandler h) {
        if (h == null) return;
        super.setConnection(h);

        SettingsFB fbPrefs = h.getSettingsFB();

        if (fbPrefs != null) fbPrefs.addObserver(this);
        h.addObserver(this);
    }

    @Override
    public boolean isDoubleBuffered() {
        return true;
    }

    @Override
    public void mouseMoved(MouseEvent evt) {
        processLocalMouseEvent(evt, true);
    }

    @Override
    public void mouseDragged(MouseEvent evt) {
        processLocalMouseEvent(evt, true);
    }

    @Override
    public void mousePressed(MouseEvent evt) {
        requestFocus();
        processLocalMouseEvent(evt, false);
    }

    @Override
    public void mouseReleased(MouseEvent evt) {
        processLocalMouseEvent(evt, false);
    }

    // Ignored events.
    @Override
    public void mouseClicked(MouseEvent evt) {
        if (hdlr != null && hdlr.getPhase() == RFBHandler.Phase.PROCESSING || isPreview) return;
        if (evt.getClickCount() != 1) return;
    }

    @Override
    public void mouseEntered(MouseEvent evt) {
        super.mouseEntered(evt);
        if (isPreview) return;
//    	hasMouse = true;
        requestFocus();
        repaint();
//    	if ( thePoolPanel != null) thePoolPanel.setAllEnabled( !hasMouse);
    }

    @Override
    public void mouseExited(MouseEvent evt) {
        super.mouseExited(evt);
        if (isPreview) return;
//    	hasMouse = false;
        repaint();
//    	if ( thePoolPanel != null) thePoolPanel.setAllEnabled( !hasMouse);
    }

    @Override
    public void mouseWheelMoved(MouseWheelEvent mwe) {
        processLocalMouseEvent(mwe, false);
    }

    private Dimension getTranslationOffset(Point pInLocalCoords) {
        double dx = getDesktopScalingFactor() * 1.0;
        double dy = dx;

        if (hdlr.getSettingsFB().wants2AutoScale()) {
            Dimension d = getSize();
            Dimension dOrigRDSize = hdlr.getRemoteDesktopOriginalSize();
            double ow = dOrigRDSize.getWidth();
            double oh = dOrigRDSize.getHeight();
            if (ow == 0) ow = d.getWidth();
            if (oh == 0) oh = d.getHeight();
            dx = getWidth() * 100 / ow;
            dy = getHeight() * 100 / oh;
        }

        if (dx == 0) dx = 1;
        if (dy == 0) dy = 1;

        double sx = (pInLocalCoords.getX() * 100 + dx / 2) / dx;
        double sy = (pInLocalCoords.getY() * 100 + dy / 2) / dy;
        return new Dimension((int) (sx - pInLocalCoords.getX()),
                (int) (sy - pInLocalCoords.getY()));
    }

    public void processLocalMouseEvent(MouseEvent evt, boolean moved) {
        if (hdlr == null || hdlr.getPhase() != RFBHandler.Phase.PROCESSING) return;

        onMouseMoved();

        Dimension hd = getTranslationOffset(evt.getPoint());
        evt.translatePoint((int) hd.getWidth(), (int) hd.getHeight());

        pMouseDbg = evt.getPoint();

        hdlr.queue(RFBConnJob.moveMouse(hdlr, evt));
    }

    @Override
    public void keyPressed(KeyEvent evt) {
        if (!hasMouse) return;
        if (evt.getID() == KeyEvent.KEY_PRESSED && evt.getKeyCode() == KeyEvent.VK_ALT) {
            if (evt.getWhen() - lastAltKey < ALT_LOCK_THRESHOLD) {
                hdlr.queue(RFBConnJob.jamAltKey(hdlr));
                // This "return" is to make sure the Alt event won't
                // be sent to the server, as the RFBHandler will generate
                // and send one anyway.
                return;
            }
            lastAltKey = evt.getWhen();
        } else {
            // Any other key will "reset" the timer. Otherwise the user may accidentally
            // trigger a jam-ALT by invoking a set of "Alt+someAppMnemonic" key presses
            // in quick succession.
            lastAltKey = -3 * ALT_LOCK_THRESHOLD;
        }

        boolean processEvent = remapPressKeyboardLocaleEvents(evt);
        if (processEvent) {
            processLocalKeyEvent(evt);
        }
    }

    @Override
    public void keyReleased(KeyEvent evt) {
        if (!hasMouse) return;

        boolean processEvent = remapReleaseKeyboardLocaleEvents(evt);
        if (processEvent) {
            processLocalKeyEvent(evt);
        }
    }

    /**
     * @param evt
     * @return true if the original event should be processed. Otherwise returns false
     */
    private boolean remapPressKeyboardLocaleEvents(KeyEvent evt) {

        switch (hdlr.getSession().getConfiguration().getUsedKeyboardLayout()) {
            case FR:
                if (evt.getModifiers() == (KeyEvent.ALT_MASK + KeyEvent.CTRL_MASK) && (evt.getKeyCode() == KeyEvent.VK_E || evt.getKeyCode() == KeyEvent.VK_DEAD_ACUTE)) {
                    KeyEvent eurEvt = new KeyEvent(evt.getComponent(), KeyEvent.KEY_PRESSED, new Date().getTime(), 0, KeyEvent.VK_EURO_SIGN, '\u20ac');
                    processLocalKeyEvent(eurEvt);
                    evt.consume();
                    return false;
                }

                if (evt.getModifiers() == (KeyEvent.ALT_MASK + KeyEvent.CTRL_MASK) && evt.getKeyCode() == KeyEvent.VK_A) {
                    KeyEvent atEvt = new KeyEvent(evt.getComponent(), KeyEvent.KEY_PRESSED, new Date().getTime(), 0, KeyEvent.VK_AT, '\u0040');
                    processLocalKeyEvent(atEvt);
                    evt.consume();
                    return false;
                }
                break;
            case US:

                if ((evt.getModifiers() == (KeyEvent.ALT_MASK) || evt.getModifiers() == (KeyEvent.ALT_MASK + KeyEvent.CTRL_MASK)) && (evt.getKeyCode() == KeyEvent.VK_E || evt.getKeyCode() == KeyEvent.VK_DEAD_ACUTE)) {
                    KeyEvent eurEvt = new KeyEvent(evt.getComponent(), KeyEvent.KEY_PRESSED, new Date().getTime(), 0, KeyEvent.VK_EURO_SIGN, '\u20ac');
                    processLocalKeyEvent(eurEvt);
                    evt.consume();
                    return false;
                }

                if ((evt.getModifiers() == (KeyEvent.ALT_MASK) || evt.getModifiers() == (KeyEvent.ALT_MASK + KeyEvent.CTRL_MASK)) && evt.getKeyCode() == KeyEvent.VK_Q) {
                    KeyEvent atEvt = new KeyEvent(evt.getComponent(), KeyEvent.KEY_PRESSED, new Date().getTime(), 0, KeyEvent.VK_AT, '\u0040');
                    processLocalKeyEvent(atEvt);
                    evt.consume();
                    return false;
                }
                break;
            default:
                if (evt.getModifiers() == (KeyEvent.ALT_MASK + KeyEvent.CTRL_MASK) && (evt.getKeyCode() == KeyEvent.VK_E || evt.getKeyCode() == KeyEvent.VK_DEAD_ACUTE)) {
                    KeyEvent eurEvt = new KeyEvent(evt.getComponent(), KeyEvent.KEY_PRESSED, new Date().getTime(), 0, KeyEvent.VK_EURO_SIGN, '\u20ac');
                    processLocalKeyEvent(eurEvt);
                    evt.consume();
                    return false;
                }

                if (evt.getModifiers() == (KeyEvent.ALT_MASK + KeyEvent.CTRL_MASK) && evt.getKeyCode() == KeyEvent.VK_Q) {
                    KeyEvent atEvt = new KeyEvent(evt.getComponent(), KeyEvent.KEY_PRESSED, new Date().getTime(), 0, KeyEvent.VK_AT, '\u0040');
                    processLocalKeyEvent(atEvt);
                    evt.consume();
                    return false;
                }

                break;
        }

        return true;
    }

    /**
     * @param evt
     * @return true if the original event should be processed. Otherwise returns false
     */
    private boolean remapReleaseKeyboardLocaleEvents(KeyEvent evt) {

        switch (hdlr.getSession().getConfiguration().getUsedKeyboardLayout()) {
            case FR:

                if (evt.getModifiers() == (KeyEvent.ALT_MASK + KeyEvent.CTRL_MASK) && (evt.getKeyCode() == KeyEvent.VK_E || evt.getKeyCode() == KeyEvent.VK_DEAD_ACUTE)) {
                    evt.consume();
                    return false;
                }

                if (evt.getModifiers() == (KeyEvent.ALT_MASK + KeyEvent.CTRL_MASK) && evt.getKeyCode() == KeyEvent.VK_A) {
                    evt.consume();
                    return false;
                }
                break;
            case US:
                if ((evt.getModifiers() == (KeyEvent.ALT_MASK) || evt.getModifiers() == (KeyEvent.ALT_MASK + KeyEvent.CTRL_MASK)) && (evt.getKeyCode() == KeyEvent.VK_E || evt.getKeyCode() == KeyEvent.VK_DEAD_ACUTE)) {
                    evt.consume();
                    return false;
                }

                if ((evt.getModifiers() == (KeyEvent.ALT_MASK) || evt.getModifiers() == (KeyEvent.ALT_MASK + KeyEvent.CTRL_MASK)) && evt.getKeyCode() == KeyEvent.VK_Q) {
                    evt.consume();
                    return false;
                }
                break;
            default:

                if (evt.getModifiers() == (KeyEvent.ALT_MASK + KeyEvent.CTRL_MASK) && (evt.getKeyCode() == KeyEvent.VK_E || evt.getKeyCode() == KeyEvent.VK_DEAD_ACUTE)) {
                    evt.consume();
                    return false;
                }

                if (evt.getModifiers() == (KeyEvent.ALT_MASK + KeyEvent.CTRL_MASK) && evt.getKeyCode() == KeyEvent.VK_Q) {
                    evt.consume();
                    return false;
                }
                break;
        }

        return true;
    }

    @Override
    public void keyTyped(KeyEvent evt) {
        if (!hasMouse) return;
    }

    private void processLocalKeyEvent(KeyEvent evt) {
        if (!hasMouse) return;
        hdlr.queue(RFBConnJob.pressKey(hdlr, evt));
        doConsume(evt);
    }

    private int getDesktopScalingFactor() {
        return hdlr.getSettingsFB().getScalingFactor();
    }

    private void doConsume(KeyEvent e) {
        e.consume();
    }

    private Point translateToDisplayCoords(Point pInRemoteCoords) {
        Dimension drDT = hdlr.getRemoteDesktopOriginalSize();

        if (getWidth() == 0 || getHeight() == 0) return new Point(0, 0);

        double dx = drDT.getWidth() / getWidth();
        double dy = drDT.getHeight() / getHeight();

        double px = dx != 0 ? pInRemoteCoords.x / dx : 0;
        double py = dy != 0 ? pInRemoteCoords.y / dy : 0;

        return new Point((int) px, (int) py);
    }

    private void onMouseMoved() {
        RemoteCursor rc = hdlr.getCursor();

        double sf = hdlr.getSettingsFB().getCursorScalingFactor() * 0.01;
        Point hs = rc.getHotSpot();
        Point prc = translateToDisplayCoords(rc.getRemotePos());
        Dimension drc = rc.getSize();

        repaint(rMouseNow);

        rMouseNow = new Rectangle((int) (prc.x - sf * hs.x), (int) (prc.y - sf * hs.y),
                (int) (drc.width * sf), (int) (drc.height * sf));

        handleRemoteCursorDisplayModeChange(this);
//        revalidate();
    }

    private Dimension compute() {
        Dimension dOrigRDSize = hdlr.getRemoteDesktopOriginalSize();
        int sf = hdlr.getSettingsFB().getScalingFactor();
        double dw = dOrigRDSize.getWidth() * sf / 100;
        double dh = dOrigRDSize.getHeight() * sf / 100;
        Dimension d = new Dimension((int) dw, (int) dh);

        return d;
    }

    @Override
    public Dimension getPreferredScrollableViewportSize() {
        return super.getPreferredSize();
    }

    @Override
    public int getScrollableBlockIncrement(Rectangle visibleRect, int orientation, int direction) {
        return 200;
    }

    @Override
    public boolean getScrollableTracksViewportHeight() {
        if (isPreview || hdlr == null) return true;
        return hdlr.getSettingsFB().wants2AutoScale();
    }

    @Override
    public boolean getScrollableTracksViewportWidth() {
        if (isPreview || hdlr == null) return true;
        return hdlr.getSettingsFB().wants2AutoScale();
    }

    @Override
    public int getScrollableUnitIncrement(Rectangle visibleRect, int orientation, int direction) {
        return 50;
    }

    @Override
    protected boolean isDisconnected() {
        return rDesktop == null || super.isDisconnected();
    }

    @Override
    protected void paintInternal(Graphics g) {
        Rectangle rDT = new Rectangle(0, 0, dblBuff.getWidth(this), dblBuff.getHeight(this));

        if (isDisconnected()) {
            showDisconnected(g, getSize(), getStatusMessage());
            return;
        }

        if (!hasMouse && showControlNavigator) {
            g.drawImage(rGraytop, rDT.x, rDT.y, rDT.width, rDT.height, this);
            showGlasspaneKeyTip(g);
        } else if (hdlr != null && hdlr.isDisabledRemoteControlling()) {
            g.drawImage(rDesktop, rDT.x, rDT.y, rDT.width, rDT.height, this);
            showDisableControlsMessage(g);
        } else {
            g.drawImage(rDesktop, rDT.x, rDT.y, rDT.width, rDT.height, this);
        }

        if (rPointer != null && hdlr.getSettingsFB().

                getCursorDisplayMode() != CursorDisplayMode.SYSTEM)
            g.drawImage(rPointer, rMouseNow.x, rMouseNow.y, rMouseNow.width, rMouseNow.height, this);

        // Some debug code to show the encoding used by the server ( Developers only)
        if (SettingsFB.showDebugInfo)

        {
            VNCEncoding enc = hdlr.getEncodingUsedByServer();
            g.setColor(Color.red);
            g.drawString(enc.toString() + " @ " + hdlr.getRemoteDesktopsBytesPerPixel() + " Bytes PP", 10, 10);

            String mp = "@( " + pMouseDbg.x + " / " + pMouseDbg.y + ")";
            g.drawString(mp, pMouseDbg.x + 30, pMouseDbg.y);
        }

        // Show a warning to the user in a prominent spot if the Alt key is "jammed" down
        if (hdlr.isAltKeyJammed())

        {
            Dimension s = getSize();
            Rectangle2D r = new Rectangle2D.Double(10, 10, s.getWidth(), s.getHeight() * 0.8);
            SharedGraphicsUtils.printDoubleBold(g, ">>> Alt Key is 'locked down' <<<", r, g.getFont(), Color.black, Color.red);
        }

    }

    /**
     * This is a cheap ass hack to avoid code duplication. It's not pretty, but it's getting the job done.
     */
    private static void handleRemoteCursorDisplayModeChange(RemoteDesktopPanel rdp) {
        RFBHandler hdlr = rdp.hdlr;
        rdp.rPointer = rdp.createImage(rdp.hdlr.getRemotePointerSource());
        CursorDisplayMode cpm = hdlr.getSettingsFB().getCursorDisplayMode();

        if (cpm == CursorDisplayMode.SYSTEM || cpm == CursorDisplayMode.PAINT_N_UPDATE_SYSTEM) {
            Toolkit tk = Toolkit.getDefaultToolkit();
            Cursor corsair = tk.createCustomCursor(rdp.rPointer, hdlr.getCursor().getHotSpot(), "RFBDPCorsair");
            rdp.setCursor(corsair);
        }
        if (cpm == CursorDisplayMode.PAINT_N_HIDE_SYSTEM) {
            int d = 4;
            int[] pix = new int[d * d];
            Arrays.fill(pix, 0);
            MemoryImageSource mis = new MemoryImageSource(d, d, pix, 0, d);
            Image blankPtr = rdp.createImage(mis);
            Toolkit tk = Toolkit.getDefaultToolkit();
            Cursor corsair = tk.createCustomCursor(blankPtr, hdlr.getCursor().getHotSpot(), "BlankCorsair");
            rdp.setCursor(corsair);
        }
    }

    @Override
    public void update(Observable o, Object arg) {

        if (o instanceof SettingsFB) {
            SettingsFB.FBProperty p = (SettingsFB.FBProperty) arg;
            if (p == SettingsFB.FBProperty.SFCURSOR ||
                    p == SettingsFB.FBProperty.CURSOR_DISPLAY_MODE)
                onMouseMoved();

            if (p == SettingsFB.FBProperty.SFDESKTOP) {
                setPreferredSize(compute());
                revalidate();
            }

            if (p == SettingsFB.FBProperty.AUTOSCALE)
                revalidate();
        }

        if (o == hdlr) {
            final RFBHandlerEvent e = (RFBHandlerEvent) arg;
            final RemoteDesktopPanel rdp = this;

            try {
                SwingUtilities.invokeAndWait(new Runnable() {
                    public void run() {
                        if (e.operation == RFBHandlerEvent.Category.PDCHANGED) {

                            if (e.isMouse) {
                                handleRemoteCursorDisplayModeChange(rdp);
                            } else {
                                producer = new FilteredImageSource(hdlr.getRemoteDesktopSource(), filter);
                                rGraytop = createImage(producer);
                                rDesktop = createImage(hdlr.getRemoteDesktopSource());
                                rDesktop.setAccelerationPriority(1);
                                setPreferredSize(compute());
                                revalidate();
                            }

                            repaint();
                        }

                        if (e.operation == RFBHandlerEvent.Category.CURSORMOVED)
                            onMouseMoved();

                        // BUGFIX! Do NOT touch! When run as an Applet under Windumb systems, this widget will will
                        // not be resized and thus properly displayed, if the connection can't be established. This
                        // statement will FORCE the Applet to recompute the widgets preferred size and consequently
                        // show it. Why would that be so important? Well ... besides the obvious optical side effects,
                        // the Applet will FAIL TO RESPOND to ANY keyboard shortcuts (i.e. the Glasspane). As I was
                        // originally planning on DROPPING the actions bar completely, that bug would otherwise render
                        // the Applet UNUSABLE.
                        if (e.operation == RFBHandlerEvent.Category.PHASECHANGED)
                            setPreferredSize(compute());

                        // A disconnect can be caused by a fatal error in the Connection. We
                        // need to repaint the desktop WITHOUT adjusting the widgets size to
                        // make sure we will be able to read the error message !
                        if (e.operation == RFBHandlerEvent.Category.PHASECHANGED ||
                                e.operation == RFBHandlerEvent.Category.KEYJAMCHANGED) repaint();
                    }
                });
            } catch (Exception ex) {
            }
        }
    }

    public void setShowControlNavigator(boolean showControlNavigator) {
        this.showControlNavigator = showControlNavigator;
    }
}
