package com.wincornixdorf.vnc.rfb;

import java.awt.event.InputEvent;
import java.awt.event.KeyEvent;
import java.awt.event.MouseEvent;
import java.awt.event.MouseWheelEvent;
import java.io.IOException;
import java.io.OutputStream;
import java.util.HashMap;
import java.util.Map;

import com.wincornixdorf.vnc.global.VNCViewer;
import com.wincornixdorf.vnc.rfb.debug.ToolBox;

/**
 * Created by jan.fogl.
 * <p>
 * A buffer for putting pointer and keyboard events before being sent. This buffer is to ensure that
 * multiple RFB events generated from a single Java Event will all be sent in a single network packet.
 * The maximum possible length is 4 modifier down events, a single key event followed by four
 * modifier up events i.e. nine key events or 72 bytes.
 */
public class RFBEventSender {

    private static boolean traceMouse = VNCViewer.isDivine() && System.getenv("TRACEMOUSE") != null;

    private static boolean traceKeybd = VNCViewer.isDivine() && System.getenv("TRACEKEYBD") != null;

    private Map<Integer, Integer> setupedModifiers = new HashMap<Integer, Integer>();

    private X11Mapper x11Mapper = new X11Mapper();

    private OutputStream outputStream;

    private byte[] eventBuf = new byte[72];

    private int eventBufLen = 0;

    private boolean jammedAlt = false;

    private boolean mouseButtonReversed = false;


    public RFBEventSender(boolean mouseButtonReversed, OutputStream outputStream) {

        this.outputStream = outputStream;
        this.mouseButtonReversed = mouseButtonReversed;

        setupedModifiers.put(InputEvent.CTRL_MASK, 0);
        setupedModifiers.put(InputEvent.SHIFT_MASK, 0);
        setupedModifiers.put(InputEvent.META_MASK, 0);
        setupedModifiers.put(InputEvent.ALT_MASK, 0);
        setupedModifiers.put(InputEvent.ALT_GRAPH_MASK, 0);
    }


    /**
     * Method writes out provided events. Be careful events array can have only 72B, modifiers included.
     * @param events
     */
    public void writeKeyEvent(KeyEvent... events) {

        for (KeyEvent evt : events) {

            int keyChar = evt.getKeyChar();
            int keyCode = evt.getKeyCode();
            int key2send = -1;

            if (traceKeybd) System.err.println(genInfo(evt));

            boolean keyPressed = (evt.getID() == KeyEvent.KEY_PRESSED);
            boolean hasLetterForKey = keyCode >= 0x41 && keyCode <= 0x5a;
            boolean escapeCtrlLetter = evt.isControlDown() && hasLetterForKey;

            key2send = x11Mapper.keycode2x11(keyCode, keyChar);

            if (escapeCtrlLetter) {
                logKeyboard("KBDTrace KBDEvs: HAH! GOTCHA!");
                key2send += 0x60;
            }

            writeKeyEventsModifiers(evt.getModifiers());

            if (key2send == KeyEvent.CHAR_UNDEFINED && keyCode != KeyEvent.VK_DELETE) {
                logKeyboard("KBDTrace KBDEvs: Dropped bogus key!");
            } else {
                writeKeyEvent(key2send, keyPressed, "SendKEY_PRESS/RELEASE [" + key2send + "]");
            }
        }

        flush();

        logKeyboard("KBDTrace KBDEvs: Probably flushing right now");

    }

    /**
     * Write a pointer event message. We may need to send modifier key
     * events around it to set the correct modifier state.
     *
     * @param evt The MouseEvent to write
     */
    public void writePointerEvent(MouseEvent evt) {

        int x = evt.getX();
        int y = evt.getY();

        if (x < 0 || y < 0) return;

        int pointerMask = 0;
        int modifiers = evt.getModifiers();

        int maskLeft = 1;
        int maskMiddle = 2;
        int maskRight = 4;
        int maskUp = 8;
        int maskDown = 16;

        boolean wheelEvent = evt.getID() == MouseEvent.MOUSE_WHEEL;
        boolean released = evt.getID() == MouseEvent.MOUSE_RELEASED;
        boolean btnLeft = false;
        boolean btnMiddle = false;
        boolean btnRight = false;

        if (!released) {
            btnLeft = (modifiers & InputEvent.BUTTON1_MASK) == InputEvent.BUTTON1_MASK;
            btnMiddle = (modifiers & InputEvent.BUTTON2_MASK) == InputEvent.BUTTON2_MASK;
            btnRight = (modifiers & InputEvent.BUTTON3_MASK) == InputEvent.BUTTON3_MASK;
        }

        if (wheelEvent) {
            MouseWheelEvent mwe = (MouseWheelEvent) evt;
            pointerMask += (mwe.getWheelRotation() < 0) ? maskUp : maskDown;
        }

        if (mouseButtonReversed) {
            maskMiddle = 4;
            maskRight = 2;
        }

        pointerMask += btnLeft ? maskLeft : 0;
        pointerMask += btnMiddle ? maskMiddle : 0;
        pointerMask += btnRight ? maskRight : 0;

        writePointerEvent(evt.getX(), evt.getY(), pointerMask, modifiers);

        if (wheelEvent) {
            writePointerEvent(evt.getX(), evt.getY(), 0, modifiers);
        }

        flush();
    }

    public void resetKeyEventModifiers() {
        writeKeyEventsModifiers(0);
        flush();
    }

    private void flush() {
        try {
            outputStream.write(eventBuf, 0, eventBufLen);
        } catch (IOException ioe) {
        }
        eventBufLen = 0;
    }

    public boolean toggleAltJam() {
        jammedAlt = !jammedAlt;
        writeKeyEvent(0xffe9, jammedAlt, "TAJ");
        //flush( os);
        return jammedAlt;
    }

    public boolean isJammedAlt() {
        return jammedAlt;
    }

    private void writePointerEvent(int x, int y, int pointerMask, int modifiers) {

        logMouse("PointerMask [" + pointerMask + "]");

        writeKeyEventsModifiers(modifiers);
        writePointerEvent(x, y, pointerMask);

        // Always release all modifiers after an "up" event
        if (pointerMask == 0) writeKeyEventsModifiers(0);

    }


    /**
     * Write key events to set the correct modifier state.
     *
     * @return True if an event had to be written, false if there where no events to write.
     */
    private boolean writeKeyEventsModifiers(int newModifiers) {

        int evtBufLenOld = eventBufLen;

        boolean ctrlEvent = resolveModifierChange(InputEvent.CTRL_MASK, newModifiers);
        boolean shiftEvent = resolveModifierChange(InputEvent.SHIFT_MASK, newModifiers);
        boolean metaEvent = resolveModifierChange(InputEvent.META_MASK, newModifiers);
        boolean altEvent = resolveModifierChange(InputEvent.ALT_MASK, newModifiers);
        boolean altgrEvent = resolveModifierChange(InputEvent.ALT_GRAPH_MASK, newModifiers);

        if (traceKeybd) {
            System.err.println("KBDTrace ModEvs: CTRL " + ToolBox.boolYN(ctrlEvent, "☑", "☐") + " SHFT " + ToolBox.boolYN(shiftEvent, "☑", "☐") +
                    " META " + ToolBox.boolYN(metaEvent, "☑", "☐") + " ALTG " + ToolBox.boolYN(altgrEvent, "☑", "☐") +
                    " ALT  " + ToolBox.boolYN(altEvent, "☑", "☐") + " Jamm " + ToolBox.boolYN(jammedAlt, "☑", "☐"));
        }


        if (ctrlEvent) {
            writeKeyEvent(0xffe3, (newModifiers & InputEvent.CTRL_MASK) != 0, "Send_Ctrl");
        }
        if (shiftEvent) {
            writeKeyEvent(0xffe1, (newModifiers & InputEvent.SHIFT_MASK) != 0, "Send_Shift");
        }
        if (metaEvent) {
            writeKeyEvent(0xffe7, (newModifiers & InputEvent.META_MASK) != 0, "Send_Meta");
        }
        if (altEvent && !jammedAlt) {
            writeKeyEvent(0xffe9, (newModifiers & InputEvent.ALT_MASK) != 0, "Send_Alt");
        }
        if (altgrEvent) {
            writeKeyEvent(0xffea, (newModifiers & InputEvent.ALT_GRAPH_MASK) != 0, "Send_AltGr");
        }

        return evtBufLenOld != eventBufLen;
    }

    /**
     * @param inputEvent
     * @param newModifiers
     * @return return true whenever change od modifier is detected.
     */
    private boolean resolveModifierChange(int inputEvent, int newModifiers) {

        if (setupedModifiers.get(inputEvent) == null) {
            return false;
        }

        if (newModifiers == 0 && setupedModifiers.get(inputEvent) != 0) {
            setupedModifiers.put(inputEvent, newModifiers);
            return true;
        }

        if ((newModifiers & inputEvent) != 0 && (setupedModifiers.get(inputEvent) & inputEvent) == 0) {
            setupedModifiers.put(inputEvent, newModifiers);
            return true;
        }

        return false;
    }


    /**
     * Writes key event into event buffer. In other words send it out.
     *
     * @param keysym
     * @param pressed
     * @param dbg
     */
    private void writeKeyEvent(int keysym, boolean pressed, String dbg) {

        logKeyboard("KBDTrace KBDEvs: writeKeyEvent( int, boolean)->[" + dbg + "] [" + pressed + "] KEYSYM: [" + Integer.toHexString(keysym) + "] (" + keysym + ")");

        eventBuf[eventBufLen++] = (byte) RFB.KeyboardEvent;
        eventBuf[eventBufLen++] = (byte) (pressed ? 1 : 0);
        eventBuf[eventBufLen++] = (byte) 0;
        eventBuf[eventBufLen++] = (byte) 0;
        eventBuf[eventBufLen++] = (byte) ((keysym >> 24) & 0xff);
        eventBuf[eventBufLen++] = (byte) ((keysym >> 16) & 0xff);
        eventBuf[eventBufLen++] = (byte) ((keysym >> 8) & 0xff);
        eventBuf[eventBufLen++] = (byte) ((keysym) & 0xff);
    }

    /**
     * Writes pointer event into event buffer. In other words send it out.
     *
     * @param x
     * @param y
     * @param pointerMask
     */
    private void writePointerEvent(int x, int y, int pointerMask) {
        eventBuf[eventBufLen++] = (byte) RFB.PointerEvent;
        eventBuf[eventBufLen++] = (byte) pointerMask;
        eventBuf[eventBufLen++] = (byte) ((x >> 8) & 0xff);
        eventBuf[eventBufLen++] = (byte) (x & 0xff);
        eventBuf[eventBufLen++] = (byte) ((y >> 8) & 0xff);
        eventBuf[eventBufLen++] = (byte) (y & 0xff);
    }

    private String genInfo(KeyEvent e) {
        if (e == null) return "Error";

        int keyCode = e.getKeyCode();
        int modi = e.getModifiers();
        int modiEx = e.getModifiersEx();

        String keyDown = e.getID() == KeyEvent.KEY_PRESSED ? "↓" : "↑";
        String kyText = keyCode != 0 ? KeyEvent.getKeyText(keyCode) : "*";
        String kyCodeHex = "0x" + Integer.toHexString(keyCode);
        String action = e.isActionKey() ? "↯" : "";
        String modiStr = KeyEvent.getKeyModifiersText(modi);
        String modiExStr = KeyEvent.getModifiersExText(modiEx);
        Character myC = e.getKeyChar() != KeyEvent.CHAR_UNDEFINED ? new Character(e.getKeyChar()) : '*';

        if (Character.isISOControl(myC)) myC = '#';

        String location = "";
        switch (e.getKeyLocation()) {
            case KeyEvent.KEY_LOCATION_STANDARD:
                location = "standard";
            case KeyEvent.KEY_LOCATION_LEFT:
                location = "left";
            case KeyEvent.KEY_LOCATION_RIGHT:
                location = "right";
            case KeyEvent.KEY_LOCATION_NUMPAD:
                location = "numpad";
            default:
                location = "unknown";
        }

        //loc: [%8$8s] [%4$1s,%5$c]
        return String.format("KBDTrace Input : %1$1s %2$1s | (%6$5d | %7$6s) = [%3$20s] = [%5$c] modi: [[%9$20s] (%10$3d)] modiEx: [[%11$20s] (%12$3d)]",
                keyDown, action, kyText, " ",
                myC, keyCode, kyCodeHex, location,
                modiStr, modi, modiExStr, modiEx);
    }


    private void logKeyboard(String message) {
        if (traceKeybd) {
            System.err.println(message);
        }
    }

    private void logMouse(String message) {
        if (traceMouse) {
            System.err.println(message);
        }
    }

}
