package com.wincornixdorf.vnc.rfb.conf;

import java.util.Observable;

/**The SettingsFB (FB = Framebuffer) class stores information on how the remote
 * framebuffer of a connection is to be displayed. Among the most important
 * values stored here are the zoom factors to be applied to the remote desktop
 * and the remote cursor when drawing them.*/
public class SettingsFB extends Observable {
    public static int MINZOOMFACTOR = 10;
    public static int MAXZOOMFACTOR = 300;
    public static int DEFZOOMFACTOR = 100;

    /**Set this one true, if you want to see additional information being
    displayed in the RemoteDesktopPanel. This is probably useful only for
    developers.*/
    public static boolean   showDebugInfo                       = false;

    private boolean         autoZoom                            = false;
    private boolean         preserveRatio                       = false; // not implemented yet
    private boolean         matchZoomFactors                    = true;
    private int             zoomFactorRDesktop                  = DEFZOOMFACTOR;
    private int             zoomFactorMousePtr                  = DEFZOOMFACTOR;
    
    // TODO: Make this choice available through the options widget. 
    /**The manner in which the remote cursor is to be displayed to the local user.*/
	private volatile CursorDisplayMode  rcPaintMode 			= CursorDisplayMode.SYSTEM;
    
    /**The Applet supports three ways to display the remote cursor:
     * <ul>
     *  <li><b>SYSTEM</b>                The local mouse cursor is set to the shape of the remote one.</li>
     *  <li><b>PAINT_N_SPARE_SYSTEM</b>  The remote cursor is painted onto the remote desktop leaving the system cursor unchanged.</li>
     *  <li><b>PAINT_N_UPDATE_SYSTEM</b> Paint cursor into RD, while hiding the local cursor.</li>
     *  <li><b>PAINT_N_HIDE_SYSTEM</b>   Paint cursor into RD, while showing RC as Local Cursor used simultaneously. (Looks rather silly.)</li>
     * </ul>*/
	public enum CursorDisplayMode {
		SYSTEM,
		PAINT_N_SPARE_SYSTEM,
		PAINT_N_UPDATE_SYSTEM,
		PAINT_N_HIDE_SYSTEM;
		
		public String getToolTip() {
			switch ( this) {
				case SYSTEM					: return "Replace System cursor";
				case PAINT_N_SPARE_SYSTEM	: return "Draw into RD";
				case PAINT_N_HIDE_SYSTEM	: return "Draw into RD and hide System cursor";
				case PAINT_N_UPDATE_SYSTEM	: return "Draw into RD and replace System cursor";
				default						: return "ERROR!!!";
			}
		}
	};

    public enum FBProperty { SHOWBUTTONS, AUTOSCALE,
        PRESERVERATIO, MATCHZOOMFACTORS, SFCURSOR, SFDESKTOP,
        DTFORMAT, DTFORMATSTRING, CURSOR_DISPLAY_MODE
    };

    public SettingsFB() { super(); }

    public CursorDisplayMode getCursorDisplayMode() { return rcPaintMode; }
    public void setCursorDisplayMode( CursorDisplayMode cpm) {
    	if ( rcPaintMode == cpm) return;
    	rcPaintMode = cpm;
    	setChanged();
    	notifyObservers( FBProperty.CURSOR_DISPLAY_MODE);
    }

    public boolean wants2AutoScale() {
        return autoZoom;
    }

    public void setAutoScale( boolean b) {
        if ( b == autoZoom) return;
        autoZoom = b;
        setChanged();
        notifyObservers( FBProperty.AUTOSCALE);
    }

    public boolean wants2PreserveRatio() {
        return preserveRatio;
    }

    public void setPreserveRatio( boolean b) {
        if ( b == preserveRatio) return;
        preserveRatio = b;
        setChanged();
        notifyObservers( FBProperty.PRESERVERATIO);
    }

    public boolean wants2MatchZoomFactors() {
        return matchZoomFactors;
    }

    public void setMatchZoomFactors( boolean b) {
        if ( b == matchZoomFactors) return;
        matchZoomFactors = b;

        if ( matchZoomFactors)
            setCursorScalingFactor( zoomFactorRDesktop);

        setChanged();
        notifyObservers( FBProperty.MATCHZOOMFACTORS);
    }

    public int getCursorScalingFactor() {
        return zoomFactorMousePtr;
    }

    public void setCursorScalingFactor( int i) {
        if ( i == zoomFactorMousePtr) return;
        zoomFactorMousePtr = i;

        if ( wants2MatchZoomFactors())
            setScalingFactor( zoomFactorMousePtr);

        setChanged();
        notifyObservers( FBProperty.SFCURSOR);
    }

    public int getScalingFactor() {
        return zoomFactorRDesktop;
    }

    public void setScalingFactor( int i) {
        if ( i < MINZOOMFACTOR || i > MAXZOOMFACTOR|| i == zoomFactorRDesktop) return;

        zoomFactorRDesktop = i;

        if ( wants2MatchZoomFactors())
            setCursorScalingFactor( zoomFactorRDesktop);

        setChanged();
        notifyObservers( FBProperty.SFDESKTOP);
    }

    public void dump() {
        System.out.println( "zoom RDesktop: " + zoomFactorRDesktop);
        System.out.println( "AutoZoom       " + autoZoom);
    }
}
