package com.wincornixdorf.vnc.rfb.conf;

import java.util.Observable;
import java.text.DateFormat;
import java.text.SimpleDateFormat;

/**The SettingsFT (FT = FileTransfer) class stores all information on how to
 * display the remote filesystem in the file transfer widget. Some of the most
 * important settings stored here are the date/time format to be used to display
 * file timestamps and the users bookmarks.*/
public class SettingsFT extends Observable {

    // String constants used to represent DateTime Locale formats to the user
    public  static String   sDTCustom                           = "Custom";
    public  static String   sDTSHORT                            = "Short";
    public  static String   sDTMEDIUM                           = "Medium";
    public  static String   sDTLONG                             = "Long";
    public  static String   sDTFULL                             = "Full";

    /**Date Time format to use when displaying timestamps in the FT widget. If
    the custom string is to be used, it has to be set to -1. Thus values in
    between -1 and 3 will be accepted.*/
    private int             dtFormat                            = -1;

    /**Custom Date Time format String. Will be used if dtFormat is -1.*/
    private String          dtCustomFormat                      = "dd.MM.yyyy HH:mm";

    private String          lastUsedClientDir                   = "";
    private String          lastUsedServerDir                   = "";

    private String[]        clientBookmarks                     = null;
    private String[]        serverBookmarks                     = null;

    public enum FTProperty { DTFORMAT, DTFORMATSTRING };


    public SettingsFT() { super(); }

    public DateFormat getDateFormat() {
        if ( usesCustomFormat()) return new SimpleDateFormat( dtCustomFormat);
        return DateFormat.getDateInstance( dtFormat);
    }

    public boolean usesCustomFormat() {
        return dtFormat == -1;
    }

    public int getDTFormat() {
        return dtFormat;
    }

    public String getDTFormatById() {
        switch ( dtFormat) {
            case DateFormat.FULL   : return sDTFULL;
            case DateFormat.LONG   : return sDTLONG;
            case DateFormat.MEDIUM : return sDTMEDIUM;
            case DateFormat.SHORT  : return sDTSHORT;
            default : return sDTCustom;
        }
    }

    public void setDTFormat( String id) {
        if ( id == null || id.isEmpty()) return;

        int i = -2;
        if ( id.equalsIgnoreCase( sDTCustom)) i = -1;
        if ( id.equalsIgnoreCase( sDTFULL))   i = DateFormat.FULL;
        if ( id.equalsIgnoreCase( sDTLONG))   i = DateFormat.LONG;
        if ( id.equalsIgnoreCase( sDTMEDIUM)) i = DateFormat.MEDIUM;
        if ( id.equalsIgnoreCase( sDTSHORT))  i = DateFormat.SHORT;
        setDTFormat( i);
    }

    public void setDTFormat( int i) {
        if ( i == dtFormat) return;

        if ( i != DateFormat.FULL   && i != DateFormat.LONG &&
             i != DateFormat.MEDIUM && i != DateFormat.SHORT && i != -1) return;

        dtFormat = i;
        setChanged();
        notifyObservers( FTProperty.DTFORMAT);
    }

    public String getCustomDTFormatString() {
        return dtCustomFormat;
    }

    public String setCustomDTFormatString( String s) {
        if ( s == null || s.isEmpty() || s.compareTo( dtCustomFormat) == 0) return "";

        try {
            new SimpleDateFormat( s);
        } catch ( Exception e) { return e.getMessage(); } 

        dtCustomFormat = s;
        setChanged();
        notifyObservers( FTProperty.DTFORMATSTRING);
        return "";
    }

    public String getPreferredClientDir() { return lastUsedClientDir; }
    public void setPreferredClientDir( String absFN) {
        lastUsedClientDir = absFN;
    }

    public String getPreferredServerDir() { return lastUsedServerDir; }
    public void setPreferredServerDir( String absFN) {
        lastUsedServerDir = absFN;
    }

    public String[] getClientBookmarks() { return clientBookmarks; }
    public void setClientBookmarks( String[] bm) {
        if ( bm != null) clientBookmarks = bm;
    }

    public String[] getServerBookmarks() { return serverBookmarks; }
    public void setServerBookmarks( String[] bm) {
        if ( bm != null) serverBookmarks= bm;
    }
}
