package com.wincornixdorf.vnc.rfb.conf;

import com.wincornixdorf.vnc.global.VNCViewerOptions;

/**
 * The VNCSessionCfg class provides a container for a number of different
 * preferences objects, which are necessary to set up a VNC Session. It should
 * not be confused with the VNCViewerOptions class which contains global settings
 * and preferences which for the most part where set through the command line or
 * the Applet parameters. The most important members are the VncServerCfg object,
 * which contains the hostname and port of the VNC server and repeater (if any) to
 * connect to, and the SettingsCN object which specifies the settings for the
 * connection. The The other members are mainly preferences settings which govern
 * the graphical representation of the remote desktop filesystem.
 */
public class VNCSessionCfg {

    /**
     * A container for storing the hostname and port of both the VNC server and
     * a VNC repeater if necessary along with the credentials for logging in.
     */
    public VNCSrvCfg vncserver = null;

    public SettingsGB gb = null;

    /**
     * Stores the connection parameters like what encoding to be using etc.
     * Since these settings have no impact on transferring files, the same
     * object is used for both the filetransfer and the framebuffer connection.
     * Ideally there should be two such parameter sets. One for each connection
     * opened in due course of a session.
     */
    public SettingsCN cn = null;

    /**
     * Settings concerning the graphical representation of the remote desktop
     * on the clients side. This object is usually passed to the RemoteDesktopPanel
     * which will then paint the remote desktop depending on these settings.
     */
    public SettingsFB fb = null;

    /**
     * Settings concerning the graphical representation of the remote filesystem
     * on the clients side. This object is usually passed to the FTWidget which
     * will display the files and directories depending on these settings.
     */
    public SettingsFT ft = null;

    /**
     * The name of a file in the clients local filesystem the viewer will then
     * record everything coming in over the socket to.
     */
    public String fnRecording = "";

    /**
     * Used internally to facilitate lazy opening the filetransfer connection.
     * Please leave this identifier alone unless you know what you are doing.
     */
    public boolean lazyOpenFTConnection = true;

    // TODO: This is a "cornered by wet paint" situation hack that had best never been conceived!
    private boolean isTrustingAllEvery1 = true;

    private KeyboardLayout usedKeyboardLayout = KeyboardLayout.US;

    /**
     * The configuration object created with this constructor will contain default
     * values for the big three setting objects: connection, framebuffer and
     * filetransfer. The vncserver and logserver members however will be null
     * to render such objects unusable until they have been configured.
     */
    // TODO: Pass SettingsGB in as constructor argument and keep global settings in Global place. NOT with a session object!
    public VNCSessionCfg() {
        gb = new SettingsGB();
        cn = new SettingsCN();
        fb = new SettingsFB();
        ft = new SettingsFT();
    }

    /**
     * The configuration object created with this constructor will mainly contain
     * default values for the big three setting objects: connection, framebuffer
     * and filetransfer. This constructor however will copy various key values
     * from the VNCViewerOptions parameter. Namely the (log) server
     * settings and the encoding to use.
     *
     * @param o The {@link VNCViewerOptions} object created at boot time.
     */
    public VNCSessionCfg(VNCViewerOptions o) {
        if (o != null) {
            vncserver = o.getVNCServerConnectionParameters();
            isTrustingAllEvery1 = o.isTrustingAllCertificates();
        }

        fnRecording = o.getFilenameRecording();

        gb = new SettingsGB( /*o*/);
        cn = new SettingsCN(o);
        fb = new SettingsFB();
        ft = new SettingsFT();
    }

    public boolean isTrustingAllCertificates() {
        return isTrustingAllEvery1;
    }

    public String toString() {
        if (vncserver == null) return "Error";
        return vncserver.toString();
    }

    public KeyboardLayout getUsedKeyboardLayout() {
        return usedKeyboardLayout;
    }

    public void setUsedKeyboardLayout(KeyboardLayout usedKeyboardLayout) {
        this.usedKeyboardLayout = usedKeyboardLayout;
    }

    public enum KeyboardLayout {
        US, DE, FR, ES
    }
}
