package com.wincornixdorf.vnc.rfb.ft;

import java.io.File;
import java.util.Date;

public class FTFile extends Object implements Comparable<FTFile> {
    private File    file            = null;
    private String  filename        = "";
    private String  filenameAlt     = "";
    // TODO: The file shouldn't have to know if it's local or remote: It's RemoteFileSystem should!
    private boolean isRemote        = false;
    private long    size            = 0;
    private Date    createdOn       = null;
    private Date    lastAccOn       = null;
    private Date    lastWrtOn       = null;
    private int     dwFileAttrib    = 0;

    private boolean isPlaceholder   = false;

    public static final FTFile PLACEHOLDER = new FTFile( true);
    
    // Constants used when dealing with M$ filesystems ...
    // I sure hope they won't be changing those anytime soon ;)
    public static int MS_READONLY              = 1;
    public static int MS_HIDDEN                = 2;
    public static int MS_SYSTEM                = 4;
    public static int MS_DIRECTORY             = 16;
    public static int MS_ARCHIVE               = 32;
    public static int MS_DEVICE                = 64;
    public static int MS_NORMAL                = 128;
    public static int MS_TEMPORARY             = 256;
    public static int MS_SPARSE_FILE           = 512;
    public static int MS_REPARSE_POINT         = 1024;
    public static int MS_COMPRESSED            = 2048;
    public static int MS_OFFLINE               = 4096;
    public static int MS_NOT_CONTENT_INDEXED   = 8192;
    public static int MS_ENCRYPTED             = 16384;

    /**This method will treat the input as a filename and try to return the root of that file. E.g. a
     * file named C:\somedir\somefile should return "C". Files like /root/somedir/somefile should return "/".
     * @param fn The filename to resolve to it's Root folder.
     * @return If the filename is not absolute, the method will return null. In the event of an error the method
     * will return an empty string. Otherwise the arguments Root folder will be returned.*/    
    public static String getDriveRoot( String fn) {
        if ( fn == null) return "";
        if ( fn.startsWith( "/")) return "/";

        int i = fn.indexOf( ":\\");
        return i < 1 ? "" : fn.substring( 0, i);
    }

    /**This one is intended for internal purposes only.*/
    public FTFile() {
    }

    /**The FTFile object created by this constructor will be marked as a
    placeholder which is supposed to represent the contents of the remote
    directory until they have been loaded. 
    @param placeholder parameter will be ignored. Method is kept for compatibility reasons only.*/
    public FTFile( boolean placeholder) {
        Date d = new Date();
        isPlaceholder = true;

        file        = null;
        filename    = "Reload or Pick Location";
        isRemote    = true;
        size        = 0;
        createdOn   = d;
        lastAccOn   = d;
        lastWrtOn   = d;
    }

    public FTFile( File f) {
        file        = f;
        filename    = new String( f.getName());
        size        = f.length();
        isRemote    = false;
        createdOn   = new Date( f.lastModified());
        lastAccOn   = new Date( f.lastModified());
        lastWrtOn   = new Date( f.lastModified());

        if ( f.isDirectory())   dwFileAttrib |= MS_DIRECTORY;
        if ( f.isFile())        dwFileAttrib |= MS_NORMAL;
        if ( f.isHidden())      dwFileAttrib |= MS_HIDDEN;
    }

    public FTFile(  String fn,
                    String fnAlternate,
                    int dwFileAttribIn,
                    long sizeIn,
                    Date tsCreate,
                    Date tsAccess,
                    Date tsWrite)
    {
        file        = null;
        filename    = fn;
        filenameAlt = fnAlternate;
        dwFileAttrib= dwFileAttribIn;
        isRemote    = true;
        size        = sizeIn;
        createdOn   = tsCreate;
        lastAccOn   = tsAccess;
        lastWrtOn   = tsWrite;
    }

    public FTFile(  String fn,
                    boolean isADirectory,
                    boolean isAFile,
                    boolean isInvisible,
                    long sizeIn,
                    Date tsCreate,
                    Date tsAccess,
                    Date tsWrite)
    {
        file        = null;
        filename    = fn;
        isRemote    = true;
        size        = sizeIn;
        createdOn   = tsCreate;
        lastAccOn   = tsAccess;
        lastWrtOn   = tsWrite;

        boolean h = isParent() ? true : isADirectory;
        if ( h)                 dwFileAttrib |= MS_DIRECTORY;
        if ( isAFile)           dwFileAttrib |= MS_NORMAL;
        if ( isInvisible)       dwFileAttrib |= MS_HIDDEN;
    }

    public File getTheFile() {
        return file;
    }

    public String getFilename() {
        return filename;
    }

    public String getAlternativeFilename() {
        return filenameAlt;
    }

    public int getFileAttr() {
        return dwFileAttrib;
    }

    public boolean canBeTransferred() {
        return( !isParent() && !servesAsPlaceholder());
    }

    public boolean canBeDeleted() {
        return( !isParent() && !servesAsPlaceholder() &&
              ( !isDirectory() || !isRemote));
    }

    public boolean isRemote() {
        return isRemote;
    }

    public boolean test( int flag) {
        return ( dwFileAttrib & flag) > 0;
    }

    public boolean isDirectory() {
        return ( dwFileAttrib & MS_DIRECTORY) > 0;
    }

    public boolean isFile() {
        // May seem silly, but I would have to examine the file attributes far more closely
        return ( !isDirectory());
    }

    public boolean isHidden() {
        return ( dwFileAttrib & MS_HIDDEN) > 0;
    }

    public boolean isParent() {
        return filename.compareTo( "..") == 0;
    }

    public boolean servesAsPlaceholder() {
        return isPlaceholder;
    }

    public long getSize() {
        return size;
    }

    public String getSizeAsString() {
        if ( isDirectory()) return( isParent() ? "Parent folder" : "Folder");
        return ( String.valueOf( size));
    }

    public String toDbgString() {
        return "FN: [" + filename + "] Size: [" + size + "]b dir: [" + isDirectory() + "] file : [" + isFile() + "]";
    }

    public String toString() {
        return filename;
    }

    public Date getCreatedOn() {
        return createdOn;
    }

    public Date getLastAccessedOn() {
        return lastAccOn;
    }

    public Date getLastWrittenToOn() {
        return lastWrtOn;
    }

    public int compareTo( FTFile o) {
        if ( isParent()) return -1;
        if ( o.isParent()) return 1;

        boolean a = isDirectory();
        boolean b = o.isDirectory();

        if ( ( a && b) || ( !a && !b))
            return( filename.compareTo( o.getFilename()));

        return ( isDirectory() ? -1 : 1);
    }
}
