package com.wincornixdorf.vnc.rfb.ft;

import java.util.regex.Pattern;
import java.util.regex.Matcher;
import java.util.regex.PatternSyntaxException;

/**A FTPath can either represent a fully qualified path (FQP) within a
file-system or a "drive" in a windumb system (which in itself is of course a FQP
too). This class was introduced to replace the old FTDrive class with a more
generic one which would facilitate introducing a bookmarking system.*/
public class FTPath extends Object implements Comparable<FTPath> {

    public enum PType {
        PATH,       /**A FQP within a file-system.*/
        FLOPPY,     /**A Node where a floppy disk gets mounted.*/
        CDROM,      /**A Node where an optical medium gets mounted.*/
        LOCAL,      /**A Node where a hard disk gets mounted.*/
        NETWORK;    /**A Node where a network FS gets mounted.*/

        /**@return A human readable, non-localised String representing the {@link PType}.*/
        public String toHRString() {
            switch ( this) {
                case PATH       : return "PATH";
                case FLOPPY     : return "Floppy Disk";
                case CDROM      : return "Optical Drive";
                case LOCAL      : return "HDD";
                case NETWORK    : return "Network Filesystem";
                default         : return "Unknown Drive Type";
            }
        }
    }

    private String      name        = "";
    private int         visits      = 0;
    private boolean     bookmarked  = false;
    private PType       type        = PType.PATH;

    /**A pattern to recognize VNC style Disk definitions.*/
    private Pattern     pVncDisk    = null;
    /**A pattern to recognize typical Windumb Drives. Only 1 char supported.*/
    private Pattern     pWinDisk    = null;

    /**The constructor will parse the argument on a best effort basis, trying
    to figure out if the argument represents a VNC style disk definition, a
    Windumb disk drive identifier or the *NIX root node. If it can't be
    identified as any of the above, it will be treated as a plain path.
    @param s A path String which must either be a Windumb DD (e.g. C:\),
             a VNC style DD (e.g. C:l), the *NIX "/" or a path.*/
    public FTPath( String s) {
        // Null will never be accepted as an argument!
        if ( s == null) {
            name = "";
            type = PType.PATH;
            return;
        }

        Pattern pVNC = getVncDisk();
        Pattern pWIN = getWinDisk();

        // If this is a VNC style Disk definition: parse it
        Matcher mVNC = pVNC != null ? pVNC.matcher( s) : null;

        if ( mVNC != null && mVNC.matches()) {
            try {
                name    = "" + s.charAt( 0) + ":\\";
                type    = byIdentifier( s.charAt( 2));
            } catch ( IndexOutOfBoundsException e) {
                name = s;
                type = PType.PATH;
            }
            return;
        }

        // Windumb drives require some magic
        Matcher mWIN = pWIN != null ? pWIN.matcher( s) : null;
        if ( mWIN.matches()) {
            name = s;
            type = guessType( s.charAt( 0));
            return;
        }

        // A *NIX style "\" path segment gets special treatment.
        if ( s.equalsIgnoreCase( "/")) {
            name = s;
            type = PType.LOCAL;
            return;
        }

        // If it is none of the above, it probably is a simple path
        name = s;
    }

    public PType getType() { return type; }

    public String getFQN() {
        return name;
    }

    public String getDisplayString() {
        if ( type == PType.PATH) return name;
        else return name + " [" + type.toHRString() + "]";
    }

    public String toString() {
        return name;
    }

    /**Use this method to get the topmost node in a path.
    @return The Root of a given Path. ( E.g. C:\foo\bar → C:\ or /home/foo/bar/ → /) or an
            empty String if there was an error.*/
    public String getRoot() {
        if ( name == null || name.isEmpty()) return "";

        // Solids are easy.
        if ( isSolid()) return name;

        // *NIX paths are highly unlikely hence I'm being sloppy here.
        if ( name.startsWith( "/")) return "/";

        // Windumb paths are expected to start with the drive ID ...
        // ... If not we return an empty String.
        String drvID = name.substring( 0, 3);
        Matcher m = getWinDisk().matcher( drvID);
        return m.matches() ? drvID : "";
    }

    public boolean isUsable() { return name != null && !name.isEmpty(); }

    public boolean isMarked() { return bookmarked; }
    public void toggleMarker() { setMarked( !bookmarked); }
    public void setMarked( boolean b) {
        bookmarked = !isSolid() && b;
    }

    public int getVisits() { return visits; }
    public void visit() { visits++; }

    public int compareTo( FTPath p) {
        return p.name.compareTo( this.name);
    }

    public boolean isOfType( char t) {
        if ( type == PType.PATH) return false;
        return byIdentifier( t) == type;
    }

    /**Checks if this path represents mount-point.
    @return Returns true if this FTPath represents a Disk.*/
    public boolean isSolid() {
        return type != PType.PATH;
    }

    private PType byIdentifier( char c) {
        switch ( c) {
            case 'p' : return( PType.PATH);
            case 'f' : return( PType.FLOPPY);
            case 'l' : return( PType.LOCAL);
            case 'c' : return( PType.CDROM);
            case 'n' : return( PType.NETWORK);
            default: return( PType.PATH);
        }
    }

    private PType guessType( char driveL) {
        switch ( Character.toUpperCase( driveL)) {
            case 'A' : return( PType.FLOPPY);
            case 'B' : return( PType.FLOPPY);
            case 'C' : return( PType.LOCAL);
            case 'D' : return( PType.LOCAL);
            case 'E' : return( PType.LOCAL);
            case 'F' : return( PType.LOCAL);
            default: return( PType.NETWORK);
        }
    }

    private Pattern getVncDisk() {
        if ( pVncDisk != null) return pVncDisk;
        try {
            pVncDisk = Pattern.compile( "[a-zA-Z]\\:[flcnp]");
        } catch ( PatternSyntaxException e) { }
        return pVncDisk;
    }

    private Pattern getWinDisk() {
        if ( pWinDisk != null) return pWinDisk;
        try {
            pWinDisk = Pattern.compile( "[a-zA-Z]:\\\\");
        } catch ( PatternSyntaxException e) { }
        return pWinDisk;
    }
}
